Package: rsofun
Title: The P-Model and BiomeE Modelling Framework
Version: 5.1.0
Authors@R: c(
            person(
              family = "Stocker",
              given = "Benjamin",
              email = "benjamin.stocker@gmail.com",
              comment = c(ORCID = "0000-0003-2697-9096"),
              role = c("aut", "cre")),
            person(
               family = "Hufkens",
               given = "Koen",
               email = "koen.hufkens@gmail.com",
               comment = c(ORCID = "0000-0002-5070-8109"),
               role = c("aut")),
            person(
               family = "Arán Paredes",
               given = "Josefa",
               email = "pepa.aran@gmail.com",
               comment = c(ORCID = "0009-0006-7176-2311"),
               role = c("aut")),
            person(
               family = "Marqués",
               given = "Laura",
               email = "lauramarqueslopez@gmail.com",
               comment = c(ORCID = "0000-0002-3593-5557"),
               role = c("ctb")),
            person(
               family = "Marcadella",
               given = "Mayeul",
               email = "mayeul.marcadella@gmail.com",
               comment = c(ORCID = "0000-0001-8555-3808"),
               role = c("ctb")),
            person(
               family = "Weng",
               given = "Ensheng",
               email = "wengensheng@gmail.com",
               comment = c(ORCID = "0000-0002-1858-4847"),
               role = c("ctb")),
            person(
              family = "Bernhard",
              given = "Fabian",
              email = "fabian.bernhard@alumni.epfl.ch",
              role = c("aut"),
              comment = c(ORCID = "0000-0003-0338-0961")),
            person(given = "Geocomputation and Earth Observation, University of Bern",
                role = c("cph", "fnd"))
           )
Description: Implements the Simulating Optimal FUNctioning framework for 
              site-scale simulations of ecosystem processes, including model 
              calibration. It contains 'Fortran 90' modules for the P-model (Stocker
              et al. (2020) <doi:10.5194/gmd-13-1545-2020>), SPLASH
              (Davis et al. (2017) <doi:10.5194/gmd-10-689-2017>)
              and BiomeE (Weng et al. (2015)
              <doi:10.5194/bg-12-2655-2015>).
URL: https://github.com/geco-bern/rsofun
BugReports: https://github.com/geco-bern/rsofun/issues
License: GPL-3
Depends: R (>= 4.1.0)
Imports: dplyr, purrr, tidyr, magrittr, GenSA, BayesianTools,
        lubridate, multidplyr, stats, utils
LazyData: true
LazyDataCompression: xz
ByteCompile: true
NeedsCompilation: yes
RoxygenNote: 7.3.2
Suggests: covr, constructive, cowplot, rcmdcheck, testthat (>= 3.0.0),
        rmarkdown, ggplot2, knitr, sensitivity, rpmodel, rlang
VignetteBuilder: knitr
Encoding: UTF-8
Config/testthat/edition: 3
Packaged: 2025-10-09 13:07:34 UTC; faber
Author: Benjamin Stocker [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2697-9096>),
  Koen Hufkens [aut] (ORCID: <https://orcid.org/0000-0002-5070-8109>),
  Josefa Arán Paredes [aut] (ORCID:
    <https://orcid.org/0009-0006-7176-2311>),
  Laura Marqués [ctb] (ORCID: <https://orcid.org/0000-0002-3593-5557>),
  Mayeul Marcadella [ctb] (ORCID:
    <https://orcid.org/0000-0001-8555-3808>),
  Ensheng Weng [ctb] (ORCID: <https://orcid.org/0000-0002-1858-4847>),
  Fabian Bernhard [aut] (ORCID: <https://orcid.org/0000-0003-0338-0961>),
  Geocomputation and Earth Observation, University of Bern [cph, fnd]
Maintainer: Benjamin Stocker <benjamin.stocker@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-09 14:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 21:29:57 UTC; unix
Archs: rsofun.so.dSYM
